unit Kontakt;

interface

uses
  RakBinaryStreamData, miscfunc,
  Controls, SysUtils, Contnrs, graphics;

type
  TKontakt = class
  private
    FVorname: string;
    FPLZ: Longint;
    FNachname: string;
    FEmail: string;
    FTelArbeit: string;
    FStrasse: string;
    FHandy: string;
    FTelPrivat: string;
    FOrt: string;
    FGeburtstag: TDate;
    FPrivat: boolean;
    FPLZNichtVersorgt: boolean;
    FGeburtstagNichtVersorgt: boolean;
    FBild: TPicture;
    function GetPLZ: Longint;
    function GetGeburtstag: TDate;
  public
    constructor Create;
    destructor Destroy; override;

    procedure LoadFromBinaryTag(Tag: TRakBinaryTag);
    procedure SaveToBinaryTag(Tag: TRakBinaryTag);

    function GetGeburtstagAsString: string;
    function SetGeburtstagAsString(Value: string): boolean;

    function GetPLZAsString: string;
    function SetPLZAsString(Value: string): boolean;

    property Vorname: string read FVorname write FVorname;
    property Nachname: string read FNachname write FNachname;
    property Strasse: string read FStrasse write FStrasse;
    property PLZNichtVersorgt: boolean read FPLZNichtVersorgt write FPLZNichtVersorgt;
    property PLZ: Longint read GetPLZ write FPLZ;
    property Ort: string read FOrt write FOrt;
    property TelPrivat: string read FTelPrivat write FTelPrivat;
    property TelArbeit: string read FTelArbeit write FTelArbeit;
    property Handy: string read FHandy write FHandy;
    property Email: string read FEmail write FEmail;
    property GeburtstagNichtVersorgt: boolean read FGeburtstagNichtVersorgt
      write FGeburtstagNichtVersorgt;
    property Geburtstag: TDate read GetGeburtstag write FGeburtstag;
    property Privat: boolean read FPrivat write FPrivat;
    property Bild: TPicture read FBild;
  end;

  TKontakte = class(TObjectList)
  private
    function GetItems(Idx: integer): TKontakt;
  public
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag);
    procedure SaveToBinaryTag(Tag: TRakBinaryTag);

    function NewKontakt: TKontakt;

    property Items[Idx: integer]: TKontakt read GetItems; default;
  end;

implementation



{ TKontakt }


constructor TKontakt.Create;
begin
  inherited;

  FPLZNichtVersorgt := true;
  FGeburtstagNichtVersorgt := true;
  FBild := TPicture.Create;
end;

destructor TKontakt.Destroy;
begin
  FBild.Free;

  inherited;
end;

function TKontakt.GetGeburtstag: TDate;
begin
  if GeburtstagNichtVersorgt then
    raise Exception.Create('Geburtstag enthlt keinen gltigen Wert.');

  Result := FGeburtstag;
end;

function TKontakt.GetGeburtstagAsString: string;
var
  Year, Month, Day: Word;
begin
  if GeburtstagNichtVersorgt then
    Result := ''
  else
  begin
    DecodeDate(Geburtstag, Year, Month, Day);
    Result := Format('%.2d.%.2d.%.4d', [Day, Month, Year]);
  end;
end;

function TKontakt.GetPLZ: Longint;
begin
  if PLZNichtVersorgt then
    raise Exception.Create('PLZ enthlt keinen gltigen Wert.');

  Result := FPLZ;
end;

function TKontakt.GetPLZAsString: string;
begin
  if PLZNichtVersorgt then
    Result := ''
  else
    Result := Format('%.5d', [PLZ]);
end;



procedure TKontakt.LoadFromBinaryTag(Tag: TRakBinaryTag);
var
  Attr: TRakBinaryAttribute;
begin
  FVorname := Tag.AttributeAsString('Vorname', '');
  FNachname := Tag.AttributeAsString('Nachname', '');
  FEmail := Tag.AttributeAsString('Email', '');
  FTelArbeit := Tag.AttributeAsString('TelArbeit', '');
  FStrasse := Tag.AttributeAsString('Strasse', '');
  FHandy := Tag.AttributeAsString('Handy', '');
  FTelPrivat := Tag.AttributeAsString('TelPrivat', '');
  FOrt := Tag.AttributeAsString('Ort', '');

  FPrivat := Tag.AttributeAsBoolean('Privat', true);

  Attr := Tag.AttributeByName('Geburtstag');
  FGeburtstagNichtVersorgt := not(Attr is TRakBinaryDateTime) or Attr.IsNull;
  if not FGeburtstagNichtVersorgt then
    FGeburtstag := TRakBinaryDateTime(Attr).Value;

  Attr := Tag.AttributeByName('PLZ');
  FPLZNichtVersorgt := not(Attr is TRakBinaryLongint) or Attr.IsNull;
  if not FPLZNichtVersorgt then
    FPLZ := TRakBinaryLongint(Attr).Value;

  Attr := Tag.AttributeByName('Bild');
  if not(Attr is TRakBinaryPicture) or Attr.IsNull then
    Bild.Graphic := nil
  else
    Bild.Graphic := TRakBinaryPicture(Attr).Graphic;
end;

procedure TKontakt.SaveToBinaryTag(Tag: TRakBinaryTag);
begin
  if Vorname <> '' then
    TRakBinaryShortString.AddToTag(Tag, 'Vorname', Vorname);
  if Nachname <> '' then
    TRakBinaryShortString.AddToTag(Tag, 'Nachname', Nachname);
  if Email <> '' then
    TRakBinaryShortString.AddToTag(Tag, 'Email', Email);
  if TelArbeit <> '' then
    TRakBinaryShortString.AddToTag(Tag, 'TelArbeit', TelArbeit);
  if Strasse <> '' then
    TRakBinaryShortString.AddToTag(Tag, 'Strasse', Strasse);
  if Handy <> '' then
    TRakBinaryShortString.AddToTag(Tag, 'Handy', Handy);
  if TelPrivat <> '' then
    TRakBinaryShortString.AddToTag(Tag, 'TelPrivat', TelPrivat);
  if Ort <> '' then
    TRakBinaryShortString.AddToTag(Tag, 'Ort', Ort);

  if not Privat then
    TRakBinaryBoolean.AddToTag(Tag, 'Privat', Privat);

  if not FGeburtstagNichtVersorgt then
    TRakBinaryDateTime.AddToTag(Tag, 'Geburtstag', Geburtstag);

  if not FPLZNichtVersorgt then
    TRakBinaryLongint.AddToTag(Tag, 'PLZ', PLZ);

  if Bild.Graphic <> nil then
    TRakBinaryPicture.AddToTag(Tag, 'Bild', Bild.Graphic);
end;

function TKontakt.SetGeburtstagAsString(Value: string): boolean;
var
  Year, Month, Day: Longint;
  sYear: string;
begin
  Result := true;
  Value := Trim(Value);
  GeburtstagNichtVersorgt := Value = '';
  if not GeburtstagNichtVersorgt then
  begin
    Result := false;
    if ListLen(Value, '.') = 3 then
    begin
      Day := StrToIntDef(ListGetAt(Value, 0, '.'), 0);
      if Day in [1..31] then
      begin
        Month := StrToIntDef(ListGetAt(Value, 1, '.'), 0);
        if Month in [1..12] then
        begin
          sYear := ListGetAt(Value, 2, '.');
          Year := StrToIntDef(sYear, -1);
          if Year >= 0 then
          begin
            if Length(sYear) < 3 then
            begin
              if Year < 30 then
                Year := 2000 + Year
              else
                Year := 1900 + Year;
            end;
            try
              Geburtstag := EncodeDate(Year, Month, Day);
              Result := true;
            except on EConvertError do;
            end;
          end;
        end;
      end;
    end;
  end;
end;

function TKontakt.SetPLZAsString(Value: string): boolean;
var
  Postleitzahl: integer;
begin
  Result := true;
  Value := Trim(Value);
  PLZNichtVersorgt := Value = '';
  if not PLZNichtVersorgt then
  begin
    Postleitzahl := StrToIntDef(Value, -1);
    Result := (Postleitzahl >= 0) and (Postleitzahl <= 99999);
    if Result then
      PLZ := Postleitzahl;
  end;
end;

{ TKontakte }

function TKontakte.GetItems(Idx: integer): TKontakt;
begin
  Result := TKontakt(inherited Items[Idx]);
end;


procedure TKontakte.LoadFromBinaryTag(Tag: TRakBinaryTag);
var
  TagList: IRakBinaryTagList;
  I: integer;
  Kontakt: TKontakt;
begin
  Clear;
  TagList := Tag.FindTagsByName('Kontakt');
  for I := 0 to TagList.Count - 1 do
  begin
    Kontakt := NewKontakt;
    Kontakt.LoadFromBinaryTag(TagList.Items(I));
  end;
end;

function TKontakte.NewKontakt: TKontakt;
begin
  Result := TKontakt.Create;
  Add(Result);
end;


procedure TKontakte.SaveToBinaryTag(Tag: TRakBinaryTag);
var
  I: integer;
begin
  for I := 0 to Count - 1 do
    Items[I].SaveToBinaryTag(Tag.AddTag('Kontakt'));
end;

end.
